"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = require("typescript");
const utils_1 = require("@typescript-eslint/utils");
const columnType_js_1 = require("../utils/columnType.js");
const treeTraversal_js_1 = require("../utils/treeTraversal.js");
const createRule = utils_1.ESLintUtils.RuleCreator((name) => `https://github.com/daniel7grant/eslint-plugin-typeorm-typescript#typeorm-typescript${name}`);
const enforceColumnTypes = createRule({
    name: 'enforce-column-types',
    defaultOptions: [{}],
    meta: {
        type: 'problem',
        docs: {
            description: 'TypeORM and TypeScript types should be the same on columns.',
        },
        hasSuggestions: true,
        messages: {
            typescript_typeorm_column_mismatch: 'Type of {{ propertyName }}{{ className }} is not matching the TypeORM column type{{ expectedValue }}.',
            typescript_typeorm_column_mismatch_weird: 'Type of {{ propertyName }}{{ className }} should be string, as decimals and bigints are encoded as strings by PostgreSQL and MySQL drivers. If you are using SQLite, change the driver option to sqlite.',
            typescript_typeorm_column_suggestion: 'Change the type of {{ propertyName }} to {{ expectedValue }}.',
        },
        schema: [
            {
                type: 'object',
                properties: {
                    driver: {
                        type: 'string',
                        enum: ['postgres', 'mysql', 'sqlite'],
                    },
                },
                additionalProperties: false,
            },
        ],
    },
    create(context) {
        return {
            PropertyDefinition(node) {
                var _a, _b, _c;
                const columnArguments = (0, treeTraversal_js_1.findEitherDecoratorArguments)(node.decorators, [
                    'Column',
                    'PrimaryColumn',
                    'PrimaryGeneratedColumn',
                    'CreateDateColumn',
                    'UpdateDateColumn',
                    'DeleteDateColumn',
                    'VersionColumn',
                ]);
                if (!columnArguments || !node.typeAnnotation) {
                    return;
                }
                const [column, colArguments] = columnArguments;
                const typeormType = (0, columnType_js_1.convertArgumentToColumnType)(column, colArguments, (_b = (_a = context.options) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.driver);
                const { typeAnnotation } = node.typeAnnotation;
                let typescriptType;
                let services;
                let checker;
                try {
                    services = utils_1.ESLintUtils.getParserServices(context);
                    checker = services.program.getTypeChecker();
                }
                catch (_d) {
                    // No typechecker found, continue with typeless
                }
                if (services && checker) {
                    const type = services.getTypeAtLocation(node);
                    const typeNode = checker.typeToTypeNode(type, undefined, typescript_1.NodeBuilderFlags.None);
                    if (typeNode) {
                        typescriptType = (0, columnType_js_1.convertTsTypeToColumnType)(typeNode, checker);
                    }
                    else {
                        typescriptType = (0, columnType_js_1.convertTypeToColumnType)(typeAnnotation);
                    }
                }
                else {
                    typescriptType = (0, columnType_js_1.convertTypeToColumnType)(typeAnnotation);
                }
                if (!(0, columnType_js_1.isTypesEqual)(typeormType, typescriptType)) {
                    const fixReplace = (0, columnType_js_1.typeToString)(typeormType, typescriptType);
                    // Construct strings for error message
                    const propertyName = ((_c = node.key) === null || _c === void 0 ? void 0 : _c.type) === utils_1.AST_NODE_TYPES.Identifier ? node.key.name : 'property';
                    const classObject = (0, treeTraversal_js_1.findParentClass)(node);
                    const className = (classObject === null || classObject === void 0 ? void 0 : classObject.id) ? ` in ${classObject.id.name}` : '';
                    const expectedValue = fixReplace ? ` (expected type: ${fixReplace})` : '';
                    // Report the error
                    context.report({
                        node,
                        messageId: 'typescript_typeorm_column_mismatch',
                        data: {
                            className,
                            propertyName,
                            expectedValue,
                        },
                        suggest: fixReplace
                            ? [
                                {
                                    messageId: 'typescript_typeorm_column_suggestion',
                                    fix: (fixer) => fixer.replaceText(typeAnnotation, fixReplace),
                                    data: {
                                        propertyName,
                                        expectedValue: fixReplace,
                                    },
                                },
                            ]
                            : undefined,
                        loc: node.loc,
                    });
                }
            },
        };
    },
});
exports.default = enforceColumnTypes;
